import { _ as _object_spread } from "@swc/helpers/_/_object_spread";
import { _ as _object_spread_props } from "@swc/helpers/_/_object_spread_props";
import { _ as _object_without_properties } from "@swc/helpers/_/_object_without_properties";
import { _ as _sliced_to_array } from "@swc/helpers/_/_sliced_to_array";
import React, { forwardRef, useCallback, useImperativeHandle, useRef, useState } from "react";
import { Input as TaroInput, View } from "@tarojs/components";
import { MaskClose } from "@nutui/icons-react-taro";
import Taro, { ENV_TYPE, getEnv } from "@tarojs/taro";
import { formatNumber } from "./utils";
import { useConfig, useRtl } from "../configprovider/index";
import { ComponentDefaults } from "../../utils/typings";
import { usePropsValue } from "../../hooks/use-props-value";
var defaultProps = _object_spread_props(_object_spread({}, ComponentDefaults), {
    type: 'text',
    name: '',
    placeholder: undefined,
    confirmType: 'done',
    align: 'left',
    required: false,
    disabled: false,
    readOnly: false,
    maxLength: 9999,
    clearable: false,
    clearIcon: null,
    formatTrigger: 'onChange',
    autoFocus: false,
    plain: false
});
export var Input = /*#__PURE__*/ forwardRef(function(props, ref) {
    var classPrefix = 'nut-input';
    var rtl = useRtl();
    var locale = useConfig().locale;
    var _ref = _object_spread({}, defaultProps, props), focus = _ref.focus, type = _ref.type, name = _ref.name, placeholder = _ref.placeholder, align = _ref.align, disabled = _ref.disabled, readOnly = _ref.readOnly, maxLength = _ref.maxLength, clearable = _ref.clearable, clearIcon = _ref.clearIcon, formatTrigger = _ref.formatTrigger, autoFocus = _ref.autoFocus, style = _ref.style, className = _ref.className, plain = _ref.plain, onChange = _ref.onChange, onFocus = _ref.onFocus, onBlur = _ref.onBlur, onClear = _ref.onClear, formatter = _ref.formatter, onClick = _ref.onClick, confirmType = _ref.confirmType, defaultValue = _ref.defaultValue, _value = _ref.value, rest = _object_without_properties(_ref, [
        "focus",
        "type",
        "name",
        "placeholder",
        "align",
        "disabled",
        "readOnly",
        "maxLength",
        "clearable",
        "clearIcon",
        "formatTrigger",
        "autoFocus",
        "style",
        "className",
        "plain",
        "onChange",
        "onFocus",
        "onBlur",
        "onClear",
        "formatter",
        "onClick",
        "confirmType",
        "defaultValue",
        "value"
    ]);
    var _usePropsValue = _sliced_to_array(usePropsValue({
        value: _value,
        defaultValue: defaultValue,
        finalValue: '',
        onChange: onChange
    }), 2), value = _usePropsValue[0], setValue = _usePropsValue[1];
    var inputRef = useRef(null);
    var _useState = _sliced_to_array(useState(false), 2), active = _useState[0], setActive = _useState[1];
    useImperativeHandle(ref, function() {
        return {
            clear: function() {
                setValue('');
            },
            focus: function() {
                var _inputRef_current;
                (_inputRef_current = inputRef.current) === null || _inputRef_current === void 0 ? void 0 : _inputRef_current.focus();
            },
            blur: function() {
                var _inputRef_current;
                (_inputRef_current = inputRef.current) === null || _inputRef_current === void 0 ? void 0 : _inputRef_current.blur();
            },
            get nativeElement () {
                return inputRef.current;
            }
        };
    });
    var inputClass = useCallback(function() {
        return [
            classPrefix,
            "".concat(disabled ? "".concat(classPrefix, "-disabled") : ''),
            readOnly ? "".concat(classPrefix, "-readonly") : '',
            "".concat(plain ? "".concat(classPrefix, "-plain") : "".concat(classPrefix, "-container"))
        ].filter(Boolean).join(' ');
    }, [
        disabled
    ]);
    var _React_useState = _sliced_to_array(React.useState(), 2), updateState = _React_useState[1];
    var forceUpdate = React.useCallback(function() {
        return updateState({});
    }, []);
    var updateValue = function(value) {
        var trigger = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 'onChange';
        var val = value;
        if (type === 'number') val = formatNumber(val, false, true);
        if (type === 'digit') val = formatNumber(val, true, true);
        if (formatter && trigger === formatTrigger) val = formatter(val);
        setValue(val);
        if (trigger !== 'onChange') {
            var eventHandler = props[trigger];
            eventHandler === null || eventHandler === void 0 ? void 0 : eventHandler(val);
        }
        forceUpdate();
    };
    var handleFocus = function(event) {
        if (Taro.getEnv() === 'WEB') {
            var val = event.target.value;
            onFocus && onFocus(val);
        } else {
            var height = (event.detail || {}).height;
            onFocus === null || onFocus === void 0 ? void 0 : onFocus(value, height);
        }
        setActive(true);
    };
    var handleInput = function(event) {
        updateValue((event.detail || event.currentTarget).value, 'onChange');
    };
    var handleBlur = function(event) {
        var val = Taro.getEnv() === 'WEB' ? event.target.value : event.detail.value;
        updateValue(val, 'onBlur');
        setTimeout(function() {
            return setActive(false);
        }, 200);
    };
    var inputType = function(type) {
        if (getEnv() === ENV_TYPE.WEB) {
            if (type === 'digit') return 'text';
            if (type === 'number') return 'tel';
        } else if (type === 'password') {
            return 'text';
        }
        return type;
    };
    var getTextAlign = function() {
        if (rtl) {
            if (align === 'right') return 'left';
            if (align === 'left') return 'right';
        }
        return align;
    };
    return /*#__PURE__*/ React.createElement(View, {
        className: "".concat(inputClass(), "  ").concat(className || ''),
        style: style,
        onClick: function(e) {
            onClick && onClick(e);
        }
    }, /*#__PURE__*/ React.createElement(TaroInput, _object_spread_props(_object_spread({}, rest), {
        name: name,
        className: "nut-input-native",
        ref: inputRef,
        style: {
            textAlign: getTextAlign()
        },
        type: inputType(type),
        password: type === 'password',
        maxlength: maxLength,
        placeholder: placeholder === undefined ? locale.placeholder : placeholder,
        placeholderClass: "".concat(classPrefix, "-placeholder"),
        disabled: disabled || readOnly,
        value: value,
        focus: autoFocus || focus,
        confirmType: confirmType,
        onBlur: handleBlur,
        onFocus: handleFocus,
        onInput: handleInput
    })), /*#__PURE__*/ React.createElement(View, {
        style: {
            display: clearable && !readOnly && active && value.length > 0 ? 'flex' : 'none',
            alignItems: 'center',
            cursor: 'pointer'
        },
        onClick: function(e) {
            e.stopPropagation();
            if (!disabled) {
                setValue('');
                onClear === null || onClear === void 0 ? void 0 : onClear('');
            }
        }
    }, clearIcon || /*#__PURE__*/ React.createElement(MaskClose, {
        className: "nut-input-clear"
    })));
});
Input.displayName = 'NutInput';
